import logging

from django.utils.deprecation import MiddlewareMixin

from DBoD.models import TenantUserGroup, dsettings
from DBoD.utils import findTUG

TENANTUSERGROUP = 'tenantusergroup'
ANON = 'anonymous'
METHOD = 'meth'

logger = logging.getLogger(__name__)

NOSESSION = [
    '/dbod/updatesessions',
    '/metrics',
]


class DBoDMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if dsettings.DEVMODE:
            # Clear complete session, log out user
            if "clearsession" in request.GET:
                request.session.flush()

            # Set Usergroup
            if "tug" in request.GET:
                tugid = request.GET["tug"]
                try:
                    tug = TenantUserGroup.objects.get(code=tugid)
                    request.session[TENANTUSERGROUP] = tug
                    request.session[ANON] = False
                except TenantUserGroup.DoesNotExist:
                    logger.error('Keine Gruppe {}, tue nichts.'.format(tugid))

        if request.path not in NOSESSION:
            logger.setLevel(logging.DEBUG)
            session = request.session
            tug = session.get(TENANTUSERGROUP)
            anon = session.get(ANON)
            meth = session.get(METHOD)
            if tug and not anon:
                logger.debug('TUG: {} via {}'.format(tug.code, meth))
            else:
                logger.debug('Remote: {}'.format(request.META.get('REMOTE_ADDR')))
                tug, anon, meth = findTUG(request=request)
                session[TENANTUSERGROUP] = tug
                session[ANON] = anon
                session[METHOD] = meth
                logger.debug('Set TUG: {} in session with {}'.format(tug.code, meth))
