import graphene
from django.db.models import Q
from graphene_django import DjangoObjectType

from .models import EResource


class ResourceType(DjangoObjectType):
    class Meta:
        model = EResource

    name = 'Resource'
    description = 'A resource'


class Query(graphene.ObjectType):
    resource = graphene.Field(ResourceType, id=graphene.String())

    def resolve_resource(self, info, id):
        return EResource.objects.get(id=id)

    resources_list = graphene.List(ResourceType, search=graphene.String(), starts=graphene.String(),
                                   first=graphene.Int(), skip=graphene.Int())

    def resolve_resources_list(self, info, search=None, starts=None, first=None, skip=None, **kwargs):
        qs = EResource.objects.all()
        count = qs.count()

        if search:
            filter = (
                    Q(description__icontains=search) |
                    Q(name__icontains=search)
            )
            qs = qs.filter(filter)

        if starts:
            filter = (
                Q(name__istartswith=starts)
            )
            qs = qs.filter(filter)

        if skip:
            qs = qs[skip:]
        if first:
            qs = qs[:first]

        return qs


class Mutation(graphene.ObjectType):
    pass


schema = graphene.Schema(query=Query)
