import ipaddress
import logging
import re
from datetime import datetime

from MySQLdb import _mysql

from DBoD.models import Tenant, TenantUserGroup, DBoDLegacy, TenantUserGroupShibbUser
from dbodruntime.models import ExplodedIPs

GUESTS = 'guests'
ANONYMOUS = 'anonymous'

logger = logging.getLogger(__name__)


def read_link_from_dbodmysql(ressource, tenant):
    dbsettings = DBoDLegacy()
    db = _mysql.connect(host=dbsettings.MYSQLHOST,
                        port=dbsettings.MYSQLPORT,
                        user=dbsettings.MYSQLUSER,
                        passwd=dbsettings.MYSQLPASSWD,
                        db=dbsettings.MYSQLDB)

    query = """SELECT url, shibUrl FROM webDB WHERE db = "{}" and org = "{}";""".format(ressource, tenant)
    db.query(query)
    r = db.store_result()
    line = r.fetch_row(maxrows=1, how=1)
    if len(line) > 0:
        link = line[0]['url']
        if link:
            link = link.decode("utf-8")
        shib = line[0]['shibUrl']
        if shib:
            shib = shib.decode("utf-8")
        else:
            shib = None
        return link, shib
    else:
        return None, None


def dbodip(ipstr: str):
    try:
        ip = ipaddress.ip_address(ipstr)
        if isinstance(ip, ipaddress.IPv4Address):
            return [(ip.compressed, False, False)]
        elif isinstance(ip, ipaddress.IPv6Address):
            return [(ip.compressed, False, True)]
    except ValueError:
        if re.match(r"1\d{1,3}\.\d{1,3}$", ipstr):
            ipstr = ipstr + '.0.0/16'
            try:
                ip = ipaddress.IPv4Network(ipstr)
                return [(ip.compressed, True, False)]
            except ValueError:
                logger.error('IP converter: {}'.format(ipstr))
        elif re.match(r"^\d{1,3}\.\d{1,3}\.\d{1,3}$", ipstr):
            ipstr = ipstr + '.0/24'
            try:
                ip = ipaddress.IPv4Network(ipstr)
                return [(ip.compressed, True, False)]
            except ValueError:
                logger.error('IP converter: {}'.format(ipstr))
        elif re.match(r"^\d{1,3}\.\d{1,3}\.\d{1,3}-\d{1,3}$", ipstr):
            match = re.match(r"^(?P<prefix>\d{1,3}\.\d{1,3})\.(?P<start>\d{1,3})-(?P<end>\d{1,3})$",
                             ipstr)
            iprange = match.groupdict()
            prefix = iprange.get('prefix')
            sseg = int(iprange.get('start'))
            eseg = int(iprange.get('end'))

            first = ipaddress.IPv4Address(prefix + "." + str(sseg) + ".0")
            last = ipaddress.IPv4Address(prefix + "." + str(eseg) + ".255")

            networks = ipaddress.summarize_address_range(first, last)
            ret = []
            for net in networks:
                ret.append((net.compressed, True, False))
            return ret
        elif re.match(r"^\d{1,3}\.\d{1,3}-\d{1,3}$", ipstr):
            match = re.match(r"^(?P<prefix>\d{1,3})\.(?P<start>\d{1,3})-(?P<end>\d{1,3})$",
                             ipstr)
            iprange = match.groupdict()
            prefix = iprange.get('prefix')
            sseg = int(iprange.get('start'))
            eseg = int(iprange.get('end'))

            first = ipaddress.IPv4Address(prefix + "." + str(sseg) + ".0.0")
            last = ipaddress.IPv4Address(prefix + "." + str(eseg) + ".255.255")

            networks = ipaddress.summarize_address_range(first, last)
            ret = []
            for net in networks:
                ret.append((net.compressed, True, False))
            return ret
        elif re.match(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}-\d{1,3}$", ipstr):
            match = re.match(
                r"^(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3})\.(?P<start>\d{1,3})-(?P<end>\d{1,3})$",
                ipstr)
            iprange = match.groupdict()
            prefix = iprange.get('prefix')
            sseg = int(iprange.get('start'))
            eseg = int(iprange.get('end'))

            if sseg == 1: sseg = 0
            if eseg == 254: eseg = 255

            first = ipaddress.IPv4Address(prefix + "." + str(sseg))
            last = ipaddress.IPv4Address(prefix + "." + str(eseg))

            networks = ipaddress.summarize_address_range(first, last)
            ret = []
            for net in networks:
                ret.append((net.compressed, True, False))
            return ret
        else:
            logger.error('No match at all: {}'.format(ipstr))
            return None


def findTUG(request):
    shib_ip = request.META.get('Shib-Identity-Provider')
    shib_group = request.META.get('affiliation')
    shib_user = request.META.get('eppn')

    ipaddr = ipaddress.IPv4Address(request.META.get('REMOTE_ADDR'))

    logger.setLevel(logging.DEBUG)
    logger.debug(
        'Auth - IP: {} | Shib-IP: {} | Shib-Group: {} | Shib-User: {}'.format(ipaddr, shib_ip, shib_group, shib_user))
    tugips = ExplodedIPs.objects.filter(ipaddress_min__lte=int(ipaddr), ipaddress_max__gte=int(ipaddr)).order_by('prio')
    tugips.filter(tenantusergroup__is_active=True)

    if tugips.count() > 1:
        for tugip in tugips:
            logger.error(
                'Nicht eindeutige IP! {} gehört zu {}.'.format(ipaddr.exploded, tugip.tenantusergroup.name))
        return tugips.first().tenantusergroup, False, 'IP'
    elif tugips.count() == 1:
        return tugips.first().tenantusergroup, False, 'IP'
    elif shib_ip and shib_group and shib_user:
        user = TenantUserGroupShibbUser.objects.filter(
            eppn=shib_user,
            tenantusergroup__shibb_affiliate=shib_group,
            tenantusergroup__is_active=True,
            tenantusergroup__shibb_nameduser=True,
            tenantusergroup__tenant__shibb=shib_ip,
            tenantusergroup__tenant__is_active=True
        )
        if user.count() > 0:
            logger.debug('Shibboleth: Benannten Nutzer gefunden!')
            return user.first().tenantusergroup, False, 'SHIB'
        else:
            tugs = TenantUserGroup.objects.filter(
                shibb_affiliate=shib_group,
                is_active=True,
                shibb_nameduser=False,
                tenant__shibb=shib_ip,
                tenant__is_active=True
            )
            if tugs.count() > 0:
                logger.debug('Shibboleth: Gruppe gefunden!')
                return tugs.first(), False, 'SHIB'
            else:
                logger.debug('Shibboleth: Daten vorhanden aber keine Nutzergruppe')
                return anonTug(), True, 'SHIB'
    else:
        return anonTug(), True, 'None'


def anonTug():
    try:
        unknownTUG = TenantUserGroup.objects.get(code__exact=ANONYMOUS)
        return unknownTUG
    except TenantUserGroup.DoesNotExist:
        unknownT = Tenant(code=ANONYMOUS, name='Guest Tenant', desc='Tenant for unknown, unindentified users',
                          created=datetime.now(), lastupdate=datetime.now(), is_active=True)
        unknownT.save()
        unknownTUG = TenantUserGroup(code=GUESTS, name='Guest Tenant Group',
                                     desc='Tenantuser for unknown, unindentified users', tenant=unknownT,
                                     created=datetime.now(), lastupdate=datetime.now(), is_active=True)
        unknownTUG.save()
        return unknownTUG
