import argparse
import configparser
import logging
import os
import re
import sys
from collections import OrderedDict
from pathlib import Path

import django

CATEGORY = 'category'
IPAUTH = 'ipauth'
REMARK = 'remark'
LINK = 'url'
DESIREDCOLOR = 'desiredcolor'
NEWWINDOW = 'newwindow'
TEMPLATE = 'template'
USETEMPLATE = 'usetemplate'
DBSERVER = 'dbserver'
GATEWAY = 'gateway'
LICENSES = 'licenses'
INFO = 'info'
VERSION = 'version'
DATABASE = 'database'
MEMBER = 'member'
DESCRIPTION = 'description'
NAME = 'name'
ORGANIZATION = 'organization'
DEFAULTIP = 'defaultip'
SIGEL = 'sigel'
DBIS = 'dbisid'
EZB = 'ezbid'
SHIB = 'entityid'
CONTACT = 'contact'
EMAIL = 'email'
FON = 'fon'
TYPE = 'type'
ADDRESS = 'address'

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DBoD-System.settings")
django.setup()

from DBoD.models import Tenant, TenantIPAuth, TenantUser, Publisher, EResource, EResourceLegacyData, EResourceDBData, \
    TenantUserGroup, TenantUserGroupIPAuth, ResourceTenantData, EResourceLinkData
from django.contrib.auth.models import User

FORMAT = '%(asctime)s %(levelname)s %(message)s'
logging.basicConfig(format=FORMAT)
logger = logging.getLogger('read-dbod')


class multidict(OrderedDict):
    _unique = 0

    def __setitem__(self, key, val):
        if isinstance(val, dict):
            self._unique += 1
            key += "|" + str(self._unique)
        OrderedDict.__setitem__(self, key.lower(), val)


def import_IBF(dbpath: str, orgpath: str, contactsfile: str):
    global code, name, desc, ipstr, sigel, dbis, t
    logger.info('------------------------- START - Mieter, Nutzer und Nutzergruppen')
    op = Path(orgpath)
    if op.is_dir():
        logger.info('Benutze Pfad: {}'.format(op.absolute()))
    else:
        logger.error('Der Pfad "{}" existiert nicht oder ist kein Ordner. Exit.'.format(op.absolute()))
        sys.exit(2)
    files = list(op.glob('*.ibf'))
    contactfile = None
    for file in files:
        logger.info('FILE: {}'.format(file.absolute()))

        if file.name == contacts:
            logger.info('Spezielle Kontaktdatei gefunden. ({})'.format(file.absolute()))
            contactfile = file
        else:
            config: configparser.ConfigParser = configparser.ConfigParser(
                defaults=None,
                dict_type=multidict,
                strict=False,
                comment_prefixes=('//', ';', '#'),
                inline_comment_prefixes=(' //',)
            )

            ibf_file = ''

            try:
                with file.open(mode="r", encoding='utf-8') as ibf_raw:
                    for line in ibf_raw:
                        clean_line = re.sub(r"(.*) //.*$", '\\1', line, 0, re.MULTILINE)
                        clean_line = clean_line.strip('\x01\n')
                        ibf_file = ibf_file + clean_line + "\n"
                config.read_string(ibf_file)
            except UnicodeDecodeError:
                with file.open(mode="r", encoding='cp1257') as ibf_raw:
                    for line in ibf_raw:
                        clean_line = re.sub(r"(.*) //.*$", '\\1', line, 0, re.MULTILINE)
                        clean_line = clean_line.strip('\x01\n')
                        ibf_file = ibf_file + clean_line + "\n"
                config.read_string(ibf_file)

            currenttenant = None
            for section in config.sections():
                if section.lower().startswith(ORGANIZATION):
                    code = config.get(section, NAME)
                    name = config.get(section, DESCRIPTION)
                    desc = ''
                    ipstr = [ip.strip().lower() for ip in config.get(section, DEFAULTIP).split(',')]
                    sigel = config.get(section, SIGEL, fallback='')
                    dbis = config.get(section, DBIS, fallback='')
                    ezb = config.get(section, EZB, fallback='')
                    shib = config.get(section, SHIB, fallback='')

                    try:
                        t = Tenant.objects.get(code__exact=code)
                        logger.info('UPDATE: Mieter "{}"'.format(code))
                        t.code = code
                        t.name = name
                        t.sigel = sigel
                        t.dbis = dbis
                        t.ezb = ezb
                        t.shibb = shib
                        t.save()
                    except Tenant.DoesNotExist as e:
                        logger.info('CREATE: Mieter "{}'.format(code))
                        t = Tenant(code=code, name=name, sigel=sigel, dbis=dbis, shibb=shib)
                        t.save()
                    finally:
                        currenttenant = t

                    TenantIPAuth.objects.filter(tenant=t).delete()
                    for dbs in ipstr:
                        tip = TenantIPAuth(tenant=t, ipstr=dbs)
                        tip.save()

                elif section.lower().startswith(CATEGORY):
                    code = config.get(section, NAME)
                    name = config.get(section, DESCRIPTION)
                    desc = ''
                    type = config.get(section, TYPE)

                    if type == 'user':
                        usertype = TenantUserGroup.USER
                    elif type == 'admin':
                        usertype = TenantUserGroup.ADMIN
                    elif type == 'test':
                        usertype = TenantUserGroup.TEST
                    else:
                        logger.error('Unbekannter Nutzertyp: {} in {}'.format(type, file.absolute()))
                        usertype = TenantUserGroup.USER

                    try:
                        tug = TenantUserGroup.objects.get(code=code)
                        tug.name = name
                        tug.desc = desc
                        tug.tenant = currenttenant
                        tug.save()
                        logger.info('CREATE: Nutzergruppe "{}'.format(tug.code))
                    except TenantUserGroup.DoesNotExist:
                        tug = TenantUserGroup(code=code,
                                              name=name,
                                              desc=desc,
                                              tenant=currenttenant,
                                              type=usertype)
                        tug.save()
                        logger.info('UPDATE: Nutzergruppe "{}"'.format(tug.code))

                elif section.lower().startswith(IPAUTH):
                    tugstr = config.get(section, CATEGORY)
                    try:
                        tug = TenantUserGroup.objects.get(code=tugstr)
                        TenantUserGroupIPAuth.objects.filter(tenantusergroup=tug).delete()
                        desc = config.get(section, NAME)
                        ipstr = [ip.strip().lower() for ip in config.get(section, ADDRESS).split(',')]
                        for dbs in ipstr:
                            if dbs.startswith('//'):
                                dbs = re.sub(r"^//[^\d]*(.*)", "\\1", dbs, 0, re.MULTILINE)
                            if dbs != '':
                                tugip = TenantUserGroupIPAuth(tenantusergroup=tug, desc=desc, ipstr=dbs)
                                tugip.save()

                    except TenantUserGroup.DoesNotExist:
                        logger.error(
                            'Die Nutzergruope "{}" in der Datei {} existiert nicht in der Datenbank.'.format(
                                tugstr, file.absolute()))
    if contactfile:
        config: configparser.ConfigParser = configparser.ConfigParser(
            defaults=None,
            dict_type=multidict,
            strict=False,
            comment_prefixes=('//', ';', '#')
        )

        try:
            config.read(contactfile, encoding='utf-8')
        except UnicodeDecodeError:
            config.read(contactfile, encoding='cp1257')

        for section in config.sections():
            if section.lower().startswith(ORGANIZATION):
                code = config.get(section, NAME)
                contact = config.get(section, CONTACT)
                name = config.get(section, CONTACT)
                name = re.sub(r'^(Herr|Frau) (.*)', "\\2", name, 0)
                email = config.get(section, EMAIL)
                phone = config.get(section, FON)
                if phone.find('@') != -1:
                    phone = None

                try:
                    t = Tenant.objects.get(code__exact=code)
                    try:
                        tu = TenantUser.objects.get(tenant=t, user__email__exact=email)
                        logger.info('UPDATE: Nutzer mit eMail "{} für Mieter "{}"'.format(email, t.code))
                        tu.user.last_name = name
                        tu.contact = contact
                        tu.tel = phone
                        tu.is_tenantadmin = True
                        tu.user.save()
                        tu.save()
                    except TenantUser.DoesNotExist as e:
                        logger.info('CREATE: Nutzer mit eMail "{}" für Mieter "{}"'.format(email, t.code))
                        u = User(username=email,
                                 last_name=name,
                                 first_name='',
                                 email=email,
                                 is_active=True,
                                 is_superuser=False)
                        u.set_unusable_password()
                        u.save()
                        tu = TenantUser(tenant=t, tel=phone, user=u)
                        tu.save()
                except Tenant.DoesNotExist as e:
                    logger.error('Kein Mieter mit Code "{}" in der Datenbank. Datei: {}'.format(
                        code, file.absolute()))
    pt, created = Tenant.objects.get_or_create(code='pool')
    if created:
        pt.name = 'Pool Tenant'
        pt.desc = 'Pool Tenant used for the pool-user.'
        pt.save()
        logger.info('CREATE: Mieter "Pool"')
    ptug, created = TenantUserGroup.objects.get_or_create(code='pool', tenant=pt)
    if created:
        ptug.name = 'Pool Tenant User'
        ptug.desc = 'A usergroup for the pool tenant'
        ptug.type = TenantUserGroup.USER
        ptug.save()
        logger.info('CREATE: Nutzergruppe "pool"')
    logger.info('------------------------- END - Mieter, Nutzer und Nutzergruppen')
    logger.info('------------------------- START - Ressourcen, Verlage')
    dbs = Path(dbpath)
    if dbs.is_dir():
        logger.info('Benutze Pfad {}'.format(dbs.absolute()))
    else:
        logger.error('Der Pfad "{}" ist nicht vorhanden oder kein Ordner.'.format(dbs.absolute()))
        sys.exit(2)
    files = list(dbs.glob('*.ibf'))
    for file in files:
        logger.info('Bearbeite Datei: {}'.format(file.absolute()))
        config: configparser.ConfigParser = configparser.ConfigParser(
            defaults=None,
            dict_type=multidict,
            strict=False,
            comment_prefixes=('//', ';', '#')
        )

        ibf_file = ''

        try:
            with file.open(mode="r", encoding='utf-8') as ibf_raw:
                for line in ibf_raw:
                    clean_line = re.sub(r"(.*) //.*$", '\\1', line, 0, re.MULTILINE)
                    clean_line = clean_line.strip('\x01\n')
                    ibf_file = ibf_file + clean_line + "\n"
            config.read_string(ibf_file)
        except UnicodeDecodeError:
            with file.open(mode="r", encoding='cp1257') as ibf_raw:
                for line in ibf_raw:
                    clean_line = re.sub(r"(.*) //.*$", '\\1', line, 0, re.MULTILINE)
                    clean_line = clean_line.strip('\x01\n')
                    ibf_file = ibf_file + clean_line + "\n"
            config.read_string(ibf_file)

        p: Publisher = None
        dbc = 0
        for section in config.sections():
            if section.lower().startswith(ORGANIZATION):
                code = config.get(section, NAME)
                name = config.get(section, DESCRIPTION, fallback='')
                desc = ''
                member = [m.strip().lower() for m in config.get(section, MEMBER).split(',')]

                try:
                    p = Publisher.objects.get(code=code)
                    logger.info('UPDATE: Verlag "{}'.format(p.code))
                except Publisher.DoesNotExist as error:
                    p = Publisher(code=code, name=name, description=desc)
                    p.save()
                    logger.info('CREATE: Verlag "{}"'.format(p.code))

                for m in member:
                    try:
                        t = Tenant.objects.get(code__exact=m)
                        p.tenants.add(t)
                        p.save()
                    except Tenant.DoesNotExist:
                        logger.error('Verlag {} referenziert einen Mieter {} der nicht in der Datenbank ist.')

            if section.lower().startswith(DATABASE):
                dbc = dbc + 1
                try:
                    code = config.get(section, NAME)
                    name = config.get(section, DESCRIPTION)
                    desc = config.get(section, REMARK, fallback=None)
                    version = config.get(section, VERSION, fallback=None)
                    info = config.get(section, INFO, fallback=None)
                    licenses = config.getint(section, LICENSES, fallback=None)
                    gateway = config.get(section, GATEWAY, fallback=None)
                    dbserver = config.get(section, DBSERVER, fallback=None)
                    use_template = config.getboolean(section, USETEMPLATE, fallback=None)
                    template = config.get(section, TEMPLATE, fallback=None)
                    new_window = config.getboolean(section, NEWWINDOW, fallback=None)
                    link = config.get(section, LINK, fallback=None)
                    restype = EResource.ResourceType.UNKNOWN
                    if config.get(section, TYPE, fallback='').strip().lower() == "free":
                        restype = EResource.ResourceType.LINK
                    elif config.get(section, REMARK, fallback='').strip().lower() == "e-book":
                        restype = EResource.ResourceType.EBOOK
                    elif dbserver != 'lic' and not None:
                        restype = EResource.ResourceType.DB

                    e = None
                    try:
                        e = EResource.objects.get(code__exact=code)
                        logger.info('UPDATE: Ressource "{}"'.format(code))
                    except EResource.DoesNotExist as error:
                        e = EResource(code=code)
                        logger.info('CREATE: Ressource "{}"'.format(code))

                    e.name = name
                    e.publisher = p
                    e.restype = restype
                    e.description = desc
                    e.version = version
                    e.licences = licenses
                    e.save()

                    el, created = EResourceLegacyData.objects.get_or_create(eresource=e)
                    if created:
                        logger.info('CREATE: Ressource Legacydata {}'.format(e.code))
                    else:
                        logger.info('UPDATE: Ressource Legacydata {}'.format(e.code))

                    el.version = version
                    el.info = info
                    el.use_template = use_template
                    el.template = template
                    el.new_window = new_window
                    el.save()

                    edb, created = EResourceDBData.objects.get_or_create(eresource=e)
                    if created:
                        logger.info('CREATE: Ressource DBData {}'.format(e.code))
                    else:
                        logger.info('UPDATE: Ressource DBData {}'.format(e.code))

                    edb.gateway = gateway
                    edb.dbserver = dbserver
                    edb.save()

                    elink, created = EResourceLinkData.objects.get_or_create(eresource=e)
                    if created:
                        logger.info('CREATE: Ressource LinkData {}'.format(e.code))
                    else:
                        logger.info('UPDATE: Ressource LinkData {}'.format(e.code))

                    elink.link = link
                    elink.save()

                    usergroups = [m.strip().lower() for m in config.get(section, CATEGORY).split(',')]
                    for usergroup in usergroups:
                        try:
                            tug = TenantUserGroup.objects.get(code=usergroup)
                            logger.info('ADD: Nutzergruppe {} zu {}'.format(tug.code, e.code))
                            e.tenantuser.add(tug)
                            e.save()
                        except TenantUserGroup.DoesNotExist:
                            logger.error('Die Nutzergruppe "{}" ist nicht in der Datenbank'.format(tug))

                    tenants = e.publisher.tenants.all()
                    for tenant in tenants:
                        tcode = tenant.code
                        if config.has_option(section, tcode + '.user') or config.has_option(section, tcode + '.limit'):
                            tuser = config.get(section, tcode + '.user', fallback=None)
                            tpwd = config.get(section, tcode + '.pwd', fallback=None)
                            tlimit = config.get(section, tcode + '.limit', fallback=None)

                            rtd, created = ResourceTenantData.objects.get_or_create(eresource=e, tenant=tenant)
                            rtd.user = tuser
                            rtd.pwd = tpwd
                            rtd.limit = tlimit
                            rtd.save()

                            if created:
                                logger.info('CREATED: Ressource-Nutzergruppen-Data {} - {}'.format(e.code, tcode))
                            else:
                                logger.info('UPDATED: Ressource-Nutzergruppen-Data {} - {}'.format(e.code, tcode))

                    tenant = Tenant.objects.get(code='pool')
                    tcode = tenant.code
                    if config.has_option(section, tcode + '.user') or config.has_option(section, tcode + '.limit'):
                        tuser = config.get(section, tcode + '.user', fallback=None)
                        tpwd = config.get(section, tcode + '.pwd', fallback=None)
                        tlimit = config.get(section, tcode + '.limit', fallback=None)

                        rtd, created = ResourceTenantData.objects.get_or_create(eresource=e, tenant=tenant)
                        rtd.user = tuser
                        rtd.pwd = tpwd
                        rtd.limit = tlimit
                        rtd.save()

                        if created:
                            logger.info('CREATED: Ressource-Nutzergruppen-Data {} - {}'.format(e.code, tcode))
                        else:
                            logger.info('UPDATED: Ressource-Nutzergruppen-Data {} - {}'.format(e.code, tcode))

                except configparser.NoOptionError as error:
                    logger.error(
                        'Die Option "{}" in der Database-Section mit Index {} in der Datei "{}" fehlt, Diese Datenbank wird nicht gespeichert.'.format(
                            error.option, dbc, file.absolute()))
    logger.info('------------------------- END - Ressourcen, Verlage')


if __name__ == '__main__':
    logger.setLevel(logging.DEBUG)

    parser = argparse.ArgumentParser(description='Read IBF')
    parser.add_argument('-db', dest='dbpath', help='Input path for databases')
    parser.add_argument('-org', dest='orgpath', help='Input path for organizations')
    parser.add_argument('-contact', dest='contact', help='Special file with contacts for orgs')
    args = parser.parse_args()

    dbpath = args.dbpath
    orgpath = args.orgpath
    contacts = args.contact

    import_IBF(dbpath=dbpath, orgpath=orgpath, contactsfile=contacts)
