import argparse
import csv
import logging
import os
from pathlib import Path

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DBoD-System.settings")
django.setup()

from dbodruntime.utils import read_citrix_sessions

FORMAT = '%(asctime)s %(levelname)s %(message)s'
logging.basicConfig(format=FORMAT)
logger = logging.getLogger('read-dbod')


def read_from_sessionfile(sessionfile: str):
    sf = Path(sessionfile)
    if sf.exists() and sf.is_file():
        logger.info('Lese Citrix Session aus {}'.format(sf.absolute()))
        with sf.open(mode='r', encoding='utf-8') as cvsfile:
            csvreader = csv.DictReader(cvsfile, delimiter=',')
            read_citrix_sessions(csvreader)
    else:
        raise FileNotFoundError('Die Datei {} ist nicht vorhanden oder keine Datei!'.format(sf.absolute()))


if __name__ == '__main__':
    logger.setLevel(logging.DEBUG)

    parser = argparse.ArgumentParser(description='Read Citrix Sessionfile')
    parser.add_argument('-f', dest='sessionfile', help='Sessionfile')
    args = parser.parse_args()

    sessionfile = args.sessionfile

    try:
        read_from_sessionfile(sessionfile=sessionfile)
    except FileNotFoundError as error:
        logger.error('Ein Fehler!', error)
