# Create your models here.
import dbsettings
from django.db import models

from DBoD.models import Tenant, EResource, TenantUserGroup, TenantUser


class DBoDSelfServiceSettings(dbsettings.Group):
    STORAGE_PATH = dbsettings.StringValue(
        help_text='Speicherpfad für Uploads',
        default='storage/'
    )

    FTP_SERVER = dbsettings.StringValue(
        help_text='FTP Serveradresse',
        default='localhost'
    )

    FTP_USER = dbsettings.StringValue(
        help_text='FTP User',
        default='ftpuser'
    )

    FTP_PASSWORD = dbsettings.StringValue(
        help_text='FTP Password',
        default='geheim'
    )

    SENDER_EMAIL = dbsettings.StringValue(
        help_text='eMail Absender Adresse',
        default='dbod@slub-dresden.de'
    )

    RECV_EMAIL = dbsettings.StringValue(
        help_text='eMail Empfänger',
        default='DBoD Team <dbod2@slub-dresden.de>'
    )

    EMAIL_PREFIX = dbsettings.StringValue(
        help_text='eMail Subjekt Vorspann',
        default='[DBoD] '
    )

    EMAIL_BASE_URL = dbsettings.StringValue(
        help_text='Basis URL für Links',
        default='http://sdvdboddevweb.slub-dresden.de:8000'
    )


dsettings = DBoDSelfServiceSettings('Einstellungen für das Mandantenportal')


class Tug2ResApplication(models.Model):
    class Meta:
        verbose_name = "Zugriffsanfrage"
        verbose_name_plural = "Zugriffsanfragen"

    tenantuser = models.ForeignKey(TenantUser, verbose_name='Nutzer', on_delete=models.CASCADE, help_text="Nutzer")
    tenant = models.ForeignKey(Tenant, verbose_name='Tenant', on_delete=models.CASCADE, help_text="Mandant")
    eresource = models.ForeignKey(EResource, verbose_name='Ressource', on_delete=models.CASCADE, help_text="Ressource")
    tenantusergroup = models.ForeignKey(TenantUserGroup, verbose_name='Nutzergruppe', on_delete=models.CASCADE,
                                        help_text="Nutzergruppe")
    text = models.TextField('Anmerkungen', help_text="Anmerkungen")

    is_new = models.BooleanField('Neu', help_text="Ist neu", default=True)
    is_approved = models.BooleanField('Genehmigt', help_text="Ist angenommen", default=False)
    is_done = models.BooleanField('Verarbeitet', help_text="Ist umgesetzt", default=False)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '{} - {}'.format(self.tenant.code, self.eresource.code)


class ContactData(models.Model):
    class Meta:
        verbose_name = 'Kontakt Daten'
        verbose_name_plural = 'Kontakt Daten'

    tenantuser = models.ForeignKey(TenantUser, verbose_name='Nutzer', on_delete=models.CASCADE, help_text="Nutzer")
    tenant = models.ForeignKey(Tenant, verbose_name='Tenant', on_delete=models.CASCADE, help_text="Mandant")

    subject = models.CharField('Betreff', max_length=255, help_text='Betreff')
    text = models.TextField('Inhalt', help_text='Inhalt')
    ipaddress = models.GenericIPAddressField('IP Adresse', help_text='IP Adresse')
    browser = models.CharField('Browser', max_length=255, help_text='Browser')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '{} - {}'.format(self.tenantuser, self.subject)


class GuestContactData(models.Model):
    class Meta:
        verbose_name = 'Gast Kontakt Daten'
        verbose_name_plural = 'Gast Kontakt Daten'

    sendername = models.CharField('Name', max_length=255, help_text='Name')
    senderemail = models.EmailField('eMail', max_length=255, help_text='eMail Adresse')
    sendertelefon = models.CharField('Telefonnummer', max_length=255, help_text='Telefonnummer', null=True, blank=True)
    subject = models.CharField('Betreff', max_length=255, help_text='Betreff')
    text = models.TextField('Inhalt', help_text='Inhalt')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '{}'.format(self.subject)


class ReportData(models.Model):
    class Meta:
        verbose_name = 'Fehler Daten'
        verbose_name_plural = 'Fehler Daten'

    tenantuser = models.ForeignKey(TenantUser, verbose_name='Nutzer', on_delete=models.CASCADE, help_text="Nutzer")
    tenant = models.ForeignKey(Tenant, verbose_name='Tenant', on_delete=models.CASCADE, help_text="Mandant")

    eresource = models.ForeignKey(EResource, verbose_name='Ressource', on_delete=models.CASCADE, help_text="Ressource",
                                  null=True, blank=True)
    tenantusergroup = models.ForeignKey(TenantUserGroup, verbose_name='Nutzergruppe', on_delete=models.CASCADE,
                                        help_text="Nutzergruppe", null=True, blank=True)

    subject = models.CharField('Betreff', max_length=255, help_text='Betreff')
    text = models.TextField('Inhalt', help_text='Inhalt')
    ipaddress = models.GenericIPAddressField('IP Adresse', help_text='IP Adresse')
    browser = models.CharField('Browser', max_length=255, help_text='Browser')
    attachment = models.FileField('Anhang', upload_to='attachments/', help_text='Anhang, wie Screenshot',
                                  blank=True, null=True)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '{} - {}'.format(self.tenantuser, self.subject)


class UploadFile(models.Model):
    class Meta:
        verbose_name = "Hochgeladene Dateie"
        verbose_name_plural = "Hochgeladene Dateien"

    tenantuser = models.ForeignKey(TenantUser, verbose_name='Nutzer', on_delete=models.CASCADE, help_text="Nutzer")
    tenant = models.ForeignKey(Tenant, verbose_name='Mandant', on_delete=models.CASCADE, help_text="Mandant")

    iname = models.CharField('Bezeichnung', max_length=200, help_text='Eingegebener Name', blank=True, null=True)
    ipub = models.CharField('Verlag (eingegeben)', max_length=200, help_text='Eingegebener Verlag', blank=True,
                            null=True)
    idesc = models.CharField('Beschreibung', max_length=500, help_text='Eingegebene Beschreibung', blank=True,
                             null=True)
    idbis = models.CharField('DBIS ID', max_length=50, help_text='DBIS ID', blank=True, null=True)
    itext = models.CharField('Freitext', max_length=1000, help_text='Installationshinweise etc', blank=True, null=True)
    iuser = models.IntegerField('Nutzerlizenzen', help_text='Anzahl gleichzeitiger Nutzer, wenn gegeben.', blank=True,
                                null=True)

    file = models.CharField('Dateiname', max_length=500, help_text='Pfad der Datei')

    tenantusergroups = models.ManyToManyField(TenantUserGroup, verbose_name='Nutzergruppen',
                                              help_text='Nutzergruppen mit Zugriff', blank=True)

    is_new = models.BooleanField('Neu', help_text="Ist neu", default=True)
    is_approved = models.BooleanField('Genehmigt', help_text="Ist angenommen", default=False)
    is_done = models.BooleanField('Verarbeitet', help_text="Ist umgesetzt", default=False)
    ftp_upload = models.BooleanField('FTP Upload', help_text="Wurde dies per FTP extern hoch geladen?")

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.file
