# Create your views here.
import logging

from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import AuthenticationForm
from django.core.mail import EmailMultiAlternatives
from django.shortcuts import render, redirect
from django.template.loader import get_template

from DBoD.models import TenantUser, Publisher, EResource, TenantUserGroup, TenantUserGroupIPAuth, \
    ResourceTenantData
from tenantservice.forms import TenantUserForm, UserForm, TenantForm, TenantUserGroupForm, AppTug2ResForm, \
    DirectUploadForm, FtpUploadForm, ContactForm, ReportForm, GuestContactForm, OwnUserForm
from tenantservice.models import Tug2ResApplication, dsettings, UploadFile, GuestContactData, ContactData, ReportData

logger = logging.getLogger(__name__)


@login_required(login_url='login')
def owndata(request):
    tenant, tenantuser, user = getuser(request)

    if request.method == 'POST':
        userform = UserForm(request.POST, instance=user)
        tenantuserform = OwnUserForm(request.POST, instance=tenantuser)

        if userform.is_valid() and tenantuserform.is_valid():
            duser = userform.save(commit=False)
            if userform.data['password']:
                duser.set_password(userform.data['password'])
            duser.save()
            tu = tenantuserform.save(commit=False)
            tu.user = duser
            tu.save()
    else:
        userform = UserForm(instance=user)
        tenantuserform = OwnUserForm(instance=tenantuser)

    context = {
        'tenantuser': tenantuser,
        'tenant': tenant,
        'user': user,
        'userform': userform,
        'tenantuserform': tenantuserform,
    }

    return render(request, 'tenantservice/owndata.html', context)


@login_required(login_url='login')
def orgdata(request):
    tenant, tenantuser, user = getuser(request)

    otherusers = TenantUser.objects.filter(tenant=tenant, user__isnull=False)
    otherusers = otherusers.exclude(user_id__exact=tenantuser.user_id)

    tugs = TenantUserGroup.objects.filter(tenant=tenant)

    if request.method == 'POST':
        tenantform = TenantForm(request.POST, instance=tenant)

        if tenantform.is_valid():
            tenantform.save()
    else:
        tenantform = TenantForm(instance=tenant)

    context = {
        'tenantuser': tenantuser,
        'tenant': tenant,
        'tenantform': tenantform,
        'otherusers': otherusers,
        'usergroups': tugs,
    }

    return render(request, 'tenantservice/orgdata.html', context)


def getuser(request):
    user = request.user
    tenantuser = TenantUser.objects.get(user=user)
    tenant = tenantuser.tenant
    return tenant, tenantuser, user


def redirecttostart(request):
    return redirect('ts_welcome')


def tenantlogout(request):
    logout(request)
    return redirect('ts_welcome')


def tenantlogin(request):
    if request.method == 'POST':
        if request.POST.get('username') and request.POST.get('password'):
            username = request.POST['username']
            password = request.POST['password']
            user = authenticate(request, username=username, password=password)
            if user is not None:
                login(request, user)
                try:
                    tenantuser = TenantUser.objects.get(user=user, is_active=True)
                    logger.info('User {} hat Tenantuser!'.format(user.username))
                except TenantUser.DoesNotExist:
                    form = AuthenticationForm(request)
                    return render(request, 'tenantservice/login.html',
                                  {'form': form,
                                   'message': 'Dieser Login hat keinen Mandaten! Er kann nur für den Login in die Administration von DBoD benutzt werden.'})

                if 'next' in request.GET:
                    return redirect(request.GET['next'])
                else:
                    return redirect('start')
            else:
                form = AuthenticationForm(request)
                return render(request, 'tenantservice/login.html',
                              {'form': form, 'message': 'Nutzername unbekannt oder Passwort falsch.'})
    else:
        form = AuthenticationForm(request)
        return render(request, 'tenantservice/login.html', {'form': form})


@login_required(login_url='login')
def usereditor(request, userid):
    tenant, tenantuser, user = getuser(request)

    etenantuser = TenantUser.objects.get(user=userid)
    euser = etenantuser.user

    if request.method == "POST":
        tenantuserform = TenantUserForm(request.POST, instance=etenantuser)
        userform = UserForm(request.POST, instance=euser)

        if tenantuserform.is_valid() and userform.is_valid():
            duser = userform.save(commit=False)
            if userform.data['password']:
                duser.set_password(userform.data['password'])
            duser.save()

            tu = tenantuserform.save(commit=False)
            tu.user = duser
            tu.save()

    else:
        tenantuserform = TenantUserForm(instance=etenantuser)
        userform = UserForm(instance=euser)

    context = {
        'tenantuser': tenantuser,
        'tenant': tenant,
        'user': user,
        'userform': userform,
        'tenantuserform': tenantuserform,
    }

    return render(request, 'tenantservice/usereditor.html', context)


@login_required(login_url='login')
def publisher(request, pubid):
    tenant, tenantuser, user = getuser(request)
    try:
        publisher = Publisher.objects.get(code=pubid, tenants__code__contains=tenant.code)
        tugs = TenantUserGroup.objects.filter(tenant=tenant)
        res = EResource.objects.filter(publisher=publisher)
        res = res.filter(tenantuser__code__in=tugs).distinct()

        return render(request, 'tenantservice/publishereditor.html', {'publisher': publisher, 'res': res})

    except Publisher.DoesNotExist:
        pass


@login_required(login_url='login')
def usergroup(request, usergroup):
    usergroup = TenantUserGroup.objects.get(code=usergroup)

    if request.method == 'POST':
        ugform = TenantUserGroupForm(instance=usergroup, data=request.POST)
        if ugform.is_valid():
            ugform.save()

        ips = TenantUserGroupIPAuth.objects.filter(tenantusergroup=usergroup)
        context = {
            'usergroup': usergroup,
            'ugform': ugform,
            'ips': ips,
        }
    else:
        ugform = TenantUserGroupForm(instance=usergroup)
        ips = TenantUserGroupIPAuth.objects.filter(tenantusergroup=usergroup)
        context = {
            'usergroup': usergroup,
            'ugform': ugform,
            'ips': ips,
        }

    return render(request, 'tenantservice/usergroup.html', context=context)


@login_required(login_url='login')
def ressources(request):
    if request.GET.get('res'):
        res = request.GET.get('res')
        return redirect('resource', resource=res)

    tenant, tenantuser, user = getuser(request)

    tugs = TenantUserGroup.objects.filter(tenant=tenant)
    res = EResource.objects.all()
    res = res.filter(tenantuser__code__in=tugs).distinct()

    publisher = Publisher.objects.filter(tenants__code__contains=tenant.code)

    context = {
        'publishers': publisher,
        'ressources': res,
    }
    return render(request, 'tenantservice/resources.html', context=context)


@login_required(login_url='login')
def resource(request, resource):
    tenant, tenantuser, user = getuser(request)

    res = EResource.objects.get(code__exact=resource)
    rtd = ResourceTenantData.objects.filter(tenant=tenant, eresource=res)
    tugs = res.tenantuser.filter(tenant=tenant)

    context = {
        'res': res,
        'rtd': rtd,
        'tugs': tugs,
    }

    return render(request, 'tenantservice/resource.html', context=context)


@login_required(login_url='login')
def contactupload(request):
    return render(request, 'tenantservice/contactupload.html')


@login_required(login_url='login')
def app_res2tug(request, resource):
    tenant, tu, user = getuser(request)

    res = EResource.objects.get(code__exact=resource)
    tugs = TenantUserGroup.objects.filter(tenant=tenant)

    if request.method == 'POST':
        form = AppTug2ResForm(data=request.POST)
        if form.is_valid():
            tug = form.cleaned_data['tenantusergroup']
            text = form.cleaned_data['text']

            app = Tug2ResApplication(tenantuser=tu, tenant=tenant, eresource=res)
            app.text = text
            app.tenantusergroup = tug
            app.is_new = True
            app.is_approved = False
            app.is_done = False
            app.save()

            sendasemail(app)

            return redirect('start')

    else:
        form = AppTug2ResForm()
        form.fields['tenantusergroup'].queryset = tugs
        return render(request, 'tenantservice/tug2res.html', context={'form': form})


def storeUpload(file):
    filename = file.name
    path = dsettings.STORAGE_PATH
    with open(path + filename, 'wb+') as destination:
        for chunk in file.chunks():
            destination.write(chunk)
        destination.close()
        return destination.name


@login_required(login_url='login')
def uploaddirect(request):
    tenant, tu, user = getuser(request)

    if request.method == 'POST':
        form = DirectUploadForm(tenant=tenant, data=request.POST, files=request.FILES)
        if form.is_valid():
            file = request.FILES['file']
            uploaded = storeUpload(file)
            if uploaded:
                uf = UploadFile(tenantuser=tu, tenant=tenant)
                uf.iname = form.cleaned_data['name']
                uf.idesc = form.cleaned_data['desc']
                uf.ipub = form.cleaned_data['publisher']
                uf.itext = form.cleaned_data['freetext']
                uf.idbis = form.cleaned_data['dbis']
                uf.iuser = form.cleaned_data['usercount']
                uf.file = uploaded
                uf.ftp_upload = False
                uf.save()
                uf.tenantusergroups.set(form.cleaned_data['tenantusers'])
                uf.save()
                sendasemail(uf)

            messages.add_message(request, messages.INFO, 'Die Datei wurde erfolgreich verarbeitet.')
            return redirect('uploadsuccess', uploadid=uf.id)
        else:
            form.add_error('file', 'Fehler beim Speichern der Datei')

    else:
        form = DirectUploadForm(tenant=tenant)

    return render(request, 'tenantservice/directupload.html', context={'form': form, 'tu': tu, 't': tenant})


@login_required(login_url='login')
def uploadftp(request):
    user = request.user
    tu = TenantUser.objects.get(user_id=user)
    tenant = tu.tenant

    if request.method == 'POST':
        form = FtpUploadForm(tenant=tenant, data=request.POST)
        if form.is_valid():
            uf = UploadFile(tenantuser=tu, tenant=tenant)
            uf.iname = form.cleaned_data['name']
            uf.idesc = form.cleaned_data['desc']
            uf.ipub = form.cleaned_data['publisher']
            uf.itext = form.cleaned_data['freetext']
            uf.idbis = form.cleaned_data['dbis']
            uf.iuser = form.cleaned_data['usercount']
            uf.file = form.cleaned_data['filename']
            uf.ftp_upload = True
            uf.save()
            uf.tenantusergroups.set(form.cleaned_data['tenantusers'])
            uf.save()
            messages.add_message(request, messages.INFO, 'Die Datei wurde erfolgreich verarbeitet.')
            return redirect('uploadsuccess', uploadid=uf.id)

    else:
        form = FtpUploadForm(tenant=tenant)

    return render(request, 'tenantservice/ftpupload.html', context={'form': form, 'tu': tu, 't': tenant})


@login_required(login_url='login')
def uploaddetails(request, uploadid):
    user = request.user
    tu = TenantUser.objects.get(user_id=user)
    tenant = tu.tenant

    context = {}

    upload = UploadFile.objects.get(id=uploadid)

    if upload.ftp_upload:
        context.update(mode='ftp')
    else:
        context.update(mode='direct')

    return render(request, 'tenantservice/uploadsuccess.html', context=context)


@login_required(login_url='login')
def contact(request):
    user = request.user
    tu = TenantUser.objects.get(user_id=user)
    tenant = tu.tenant

    if request.method == 'POST':
        form = ContactForm(request.POST)
        if form.is_valid():
            form.save(commit=False)
            form.instance.tenant = tenant
            form.instance.tenantuser = tu
            form.instance.ipaddress = request.META.get('REMOTE_ADDR')
            form.instance.browser = request.META.get('HTTP_USER_AGENT')
            form.save()
            sendasemail(form.instance)
            return redirect('start')
    else:
        form = ContactForm()

    return render(request, 'tenantservice/contact.html', context={'form': form})


@login_required(login_url='login')
def report(request, res=None):
    user = request.user
    tu = TenantUser.objects.get(user_id=user)
    tenant = tu.tenant

    ip = request.META.get('REMOTE_ADDR')
    browser = request.META.get('HTTP_USER_AGENT')

    if request.method == 'POST':
        form = ReportForm(tenant, data=request.POST, files=request.FILES)
        if form.is_valid():
            form.save(commit=False)
            form.instance.tenant = tenant
            form.instance.tenantuser = tu
            form.instance.ipaddress = ip
            form.instance.browser = browser
            form.instance.eresource = form.cleaned_data['eresource']
            form.save()
            sendasemail(form.instance)
            return redirect('start')
    else:
        if res:
            form = ReportForm(tenant)
            form.fields['eresource'].initial = res
        else:
            form = ReportForm(tenant)

    return render(request, 'tenantservice/report.html', context={'form': form})


def guestcontact(request):
    if request.method == 'POST':
        form = GuestContactForm(request.POST)
        if form.is_valid():
            form.save()
            sendasemail(form.instance)
            return redirect('home')
    else:
        form = GuestContactForm()

    return render(request, 'tenantservice/guestcontact.html', context={'form': form})


def sendasemail(obj):
    set = dsettings
    recipients = [set.RECV_EMAIL]

    if isinstance(obj, GuestContactData):
        subject = '{} {}'.format(set.EMAIL_PREFIX, obj.subject)
        texttemplate = get_template('guestcontactmail.txt.jinja')
        htmltemplate = get_template('guestcontactmail.html.jinja')
        bodytxt = texttemplate.render(context={
            'subject': obj.subject,
            'name': obj.sendername,
            'email': obj.senderemail,
            'telefon': obj.sendertelefon,
            'text': obj.text,
            'date': obj.created
        })
        bodyhtml = htmltemplate.render(context={
            'subject': obj.subject,
            'name': obj.sendername,
            'email': obj.senderemail,
            'telefon': obj.sendertelefon,
            'text': obj.text,
            'date': obj.created
        })
        sender = '{} <{}>'.format(obj.sendername, obj.senderemail)
        email = EmailMultiAlternatives(from_email=sender, to=recipients, subject=subject, body=bodytxt)
        email.attach_alternative(bodyhtml, "text/html")
        return email.send(fail_silently=False)
    elif isinstance(obj, ContactData):
        subject = '{} {}'.format(set.EMAIL_PREFIX, obj.subject)
        texttemplate = get_template('contactmail.txt.jinja')
        htmltemplate = get_template('contactmail.html.jinja')
        bodytxt = texttemplate.render(context={
            'subject': obj.subject,
            'name': obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
            'tenantuser': obj.tenantuser,
            'tenant': obj.tenant,
            'ipaddress': obj.ipaddress,
            'browser': obj.browser,
            'text': obj.text,
            'date': obj.created
        })
        bodyhtml = htmltemplate.render(context={
            'subject': obj.subject,
            'name': obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
            'tenantuser': obj.tenantuser,
            'tenant': obj.tenant,
            'ipaddress': obj.ipaddress,
            'browser': obj.browser,
            'text': obj.text,
            'date': obj.created
        })
        sender = '{} <{}}>'.format(obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
                                   obj.tenantuser.user.email)
        email = EmailMultiAlternatives(from_email=sender, to=recipients, subject=subject, body=bodytxt)
        email.attach_alternative(bodyhtml, "text/html")
        return email.send(fail_silently=False)
    elif isinstance(obj, ReportData):
        subject = '{}{}'.format(set.EMAIL_PREFIX, obj.subject)
        texttemplate = get_template('reportmail.txt.jinja')
        htmltemplate = get_template('reportmail.html.jinja')
        bodytxt = texttemplate.render(context={
            'subject': obj.subject,
            'name': obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
            'tenantuser': obj.tenantuser,
            'tenant': obj.tenant,
            'ipaddress': obj.ipaddress,
            'browser': obj.browser,
            'res': obj.eresource,
            'tug': obj.tenantusergroup,
            'text': obj.text,
            'attachment': obj.attachment,
            'date': obj.created
        })
        bodyhtml = htmltemplate.render(context={
            'subject': obj.subject,
            'name': obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
            'tenantuser': obj.tenantuser,
            'tenant': obj.tenant,
            'ipaddress': obj.ipaddress,
            'browser': obj.browser,
            'res': obj.eresource,
            'tug': obj.tenantusergroup,
            'text': obj.text,
            'attachment': obj.attachment,
            'date': obj.created
        })
        sender = '{} <{}>'.format(obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
                                  obj.tenantuser.user.email)
        email = EmailMultiAlternatives(from_email=sender, to=recipients, subject=subject, body=bodytxt)
        email.attach_alternative(bodyhtml, "text/html")
        if obj.attachment:
            email.attach_file(obj.attachment.path)
        return email.send(fail_silently=False)
    elif isinstance(obj, Tug2ResApplication):
        texttemplate = get_template('accessapp.txt.jinja')
        htmltemplate = get_template('accessapp.html.jinja')
        subject = 'Neue Zugriffsanfrage'
        bodytxt = texttemplate.render(context={
            'subject': subject,
            'name': obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
            'tenantuser': obj.tenantuser,
            'tenant': obj.tenant,
            'text': obj.text,
            'baseurl': set.EMAIL_BASE_URL,
            'appid': obj.id,
            'res': obj.eresource,
            'tug': obj.tenantusergroup,
            'date': obj.created
        })
        bodyhtml = htmltemplate.render(context={
            'subject': subject,
            'name': obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
            'tenantuser': obj.tenantuser,
            'tenant': obj.tenant,
            'text': obj.text,
            'baseurl': set.EMAIL_BASE_URL,
            'appid': obj.id,
            'res': obj.eresource,
            'tug': obj.tenantusergroup,
            'date': obj.created
        })
        sender = '{} <{}>'.format(obj.tenantuser.user.first_name + ' ' + obj.tenantuser.user.last_name,
                                  obj.tenantuser.user.email)
        email = EmailMultiAlternatives(from_email=sender, to=recipients, subject=subject, body=bodytxt)
        email.attach_alternative(bodyhtml, "text/html")
        return email.send(fail_silently=False)
    else:
        return


@login_required(login_url='login')
def welcome(request):
    tenant, tenantuser, user = getuser(request)

    context = {
        'tenantuser': tenantuser,
        'tenant': tenant,
    }
    return render(request, 'tenantservice/welcome.html', context=context)
